#!/usr/local/bin/python

from PyLR.Parsers import GrammarParser
import PyLR, sys, getopt


class ParserParser(GrammarParser):

    def __init__(self):
	GrammarParser.__init__(self)
	self.result = [] # to be populated with productions
	self.funcmap = {}
	self.usercode = ""
	self.lexdef = ""
	self.classname = "MyParser"

    def idlistID(self, id):
	"idlist -> id"
	return [self.lexer.tokenmap.get(id, id)]

    def singletolist(self, el):
	"rhslist -> rhs"
	return [el]

    def idl_idlistID(self, l, el):
	"idlist -> idlist id"
	l.append(self.lexer.tokenmap.get(el, el))
	return l

    def rhs_idlist(self, l):
	"rhs -> idlist"
	return l

    def rhs_idlist_func(self, l, lp, id, rp):
	"rhs -> idlist LPAREN ID RPAREN"
	self.funcmap[tuple(l)] = id
	return l

    def rhslist_OR_rhs(self, l, OR, el):
	"rhs -> rhslist OR rhs"
	l.append(el)
	return l

    def lhsdef(self, lhs, COLON, rhslist, SCOLON):
	"lhsdef -> ID COLON rhslist SCOLON"
	for rhs in rhslist:
	    self.result.append(PyLR.Grammar.Production(lhs, rhs))
	return None

    def lexdef(self, ld):
	self.lexdef = ld

    def addcode(self, code):
	self.usercode = self.usercode + code

    def classname(self, name):
	self.classname = name

    def parse(self, text, outf):
	PyLR.Parser.Parser.parse(self, text)
	toks = [""] * len(self.lexer.tokenmap.keys())
	for t in self.lexer.tokenmap.keys():
	    toks[self.lexer.tokenmap[t]] = t
	g = PyLR.Grammar.LALRGrammar(self.result, toks)
	for p in g.productions:
	    funcname = self.funcmap.get(tuple(p.RHS), "unspecified")
	    p.setfuncname(funcname)
	g.extrasource = self.usercode
	print "done parsing, about to start tranlation to file tstout.py"
	if self.lexdef:
	    g.writefile(outf, self.classname, self.lexdef)
	else:
	    g.writefile(outf, self.classname)
	print "done"


def main():
    usage = "pgen.py infile outfile"
    args = sys.argv[1:]
    if not args or len(args) > 2:
	print usage
	sys.exit(0)
    inf = args[0]
    outf = args[1]
    if inf == "-":
	f = sys.stdin
    else:
	f = open(inf)
    pspec = f.read()
    f.close()
    pp = ParserParser()
    pp.parse(pspec, outf)


if __name__ == "__main__":
    main()

